@@ -32,6 +32,7 @@
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-link.h"
+#include "nautilus-marshal.h"
 #include "nautilus-metafile.h"
 #include "nautilus-search-uri.h"
 #include <eel/eel-glib-extensions.h>
@@ -40,6 +41,7 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-mime-monitor.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libxml/parser.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -93,6 +95,13 @@ typedef struct {
 	Request request;
 } Monitor;
 
+typedef struct {
+	NautilusDirectory *directory;
+	NautilusInfoProvider *provider;
+	NautilusOperationHandle *handle;
+	NautilusOperationResult result;
+} InfoProviderResponse;
+
 typedef gboolean (* RequestCheck) (const Request *);
 typedef gboolean (* FileCheck) (NautilusFile *);
 
@@ -103,6 +112,9 @@ static GHashTable *waiting_directories;
 static GHashTable *async_jobs;
 #endif
 
+/* Hide kde trashcan directory */
+static char *kde_trash_dir_name = NULL;
+
 /* Forward declarations for functions that need them. */
 static void     deep_count_load             (NautilusDirectory      *directory,
 					     const char             *uri);
@@ -116,15 +128,27 @@ static void     link_info_done              (NautilusDirectory      *directory,
 					     NautilusFile           *file,
 					     const char             *uri,
 					     const char             *name, 
-					     const char             *icon);
+					     const char             *icon,
+					     gulong                  drive_id,
+					     gulong                  volume_id);
 static gboolean file_needs_high_priority_work_done (NautilusDirectory *directory,
 						    NautilusFile      *file);
 static gboolean file_needs_low_priority_work_done  (NautilusDirectory *directory,
 						    NautilusFile      *file);
+static gboolean file_needs_extension_work_done  (NautilusDirectory *directory,
+						 NautilusFile      *file);
 static void     move_file_to_low_priority_queue    (NautilusDirectory *directory,
 						    NautilusFile      *file);
+static void     move_file_to_extension_queue    (NautilusDirectory *directory,
+						 NautilusFile      *file);
 
 
+void
+nautilus_set_kde_trash_name (const char *trash_dir)
+{
+	g_free (kde_trash_dir_name);
+	kde_trash_dir_name = g_strdup (trash_dir);
+}
 
 /* Some helpers for case-insensitive strings.
  * Move to nautilus-glib-extensions?
@@ -530,6 +554,10 @@ nautilus_directory_set_up_request (Request *request,
 		request->file_info = TRUE;
 		request->link_info = TRUE;
 	}
+	
+	if (file_attributes & NAUTILUS_FILE_ATTRIBUTE_VOLUMES) {
+		request->link_info = TRUE;
+	}
 
 	if (file_attributes & NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME) {
 		request->file_info = TRUE;
@@ -555,6 +583,8 @@ nautilus_directory_set_up_request (Request *request,
 	
 	request->metafile |= (file_attributes & 
 			      NAUTILUS_FILE_ATTRIBUTE_METADATA) != FALSE;
+	
+	request->extension_info = (file_attributes & NAUTILUS_FILE_ATTRIBUTE_EXTENSION_INFO) != FALSE;
 }
 
 static void
@@ -1718,7 +1748,7 @@ lacks_link_info (NautilusFile *file)
 			should_look_for_dot_directory_file (file)) {
 			return TRUE;
 		} else {
-			link_info_done (file->details->directory, file, NULL, NULL, NULL);
+			link_info_done (file->details->directory, file, NULL, NULL, NULL, 0, 0);
 			return FALSE;
 		}
 	} else {
@@ -1732,6 +1762,17 @@ wants_link_info (const Request *request)
 	return request->link_info;
 }
 
+static gboolean
+lacks_extension_info (NautilusFile *file)
+{
+	return file->details->pending_info_providers != NULL;
+}
+
+static gboolean
+wants_extension_info (const Request *request)
+{
+	return request->extension_info;
+}
 
 static gboolean
 has_problem (NautilusDirectory *directory, NautilusFile *file, FileCheck problem)
@@ -2045,6 +2086,15 @@ start_monitoring_file_list (NautilusDirectory *directory)
 
 	read_dot_hidden_file (directory);
 	
+	/* Hack to work around kde trash dir */
+	if (kde_trash_dir_name != NULL && nautilus_directory_is_desktop_directory (directory)) {
+		char *escaped;
+		escaped = gnome_vfs_escape_string (kde_trash_dir_name);
+		g_hash_table_insert (directory->details->hidden_file_hash,
+				     escaped, escaped);
+	}
+
+	
 #ifdef DEBUG_LOAD_DIRECTORY
 	g_message ("load_directory called to monitor file list of %s", directory->details->uri);
 #endif	
@@ -2869,8 +2919,14 @@ link_info_done (NautilusDirectory *directory,
 		NautilusFile *file,
 		const char *uri,
 		const char *name, 
-		const char *icon)
+		const char *icon,
+		gulong drive_id,
+		gulong volume_id)
 {
+	GnomeVFSVolumeMonitor *monitor;
+	GnomeVFSVolume *volume;
+	GnomeVFSDrive *drive;
+	
 	file->details->link_info_is_up_to_date = TRUE;
 
 	file->details->got_link_info = TRUE;
@@ -2882,6 +2938,22 @@ link_info_done (NautilusDirectory *directory,
 	file->details->custom_icon = g_strdup (icon);
  	nautilus_file_clear_cached_display_name (file);
 
+	volume = NULL;
+	if (volume_id != 0) {
+		monitor = gnome_vfs_get_volume_monitor ();
+		volume = gnome_vfs_volume_monitor_get_volume_by_id (monitor, volume_id);
+	}
+	nautilus_file_set_volume (file, volume);
+	gnome_vfs_volume_unref (volume);
+	
+	drive = NULL;
+	if (drive_id != 0) {
+		monitor = gnome_vfs_get_volume_monitor ();
+		drive = gnome_vfs_volume_monitor_get_drive_by_id (monitor, drive_id);
+	}
+	nautilus_file_set_drive (file, drive);
+	gnome_vfs_drive_unref (drive);
+	
 	nautilus_directory_async_state_changed (directory);
 }
 
@@ -2896,7 +2968,9 @@ static void
 link_info_read_done (NautilusDirectory *directory,
 		     const char *uri,
 		     const char *name,
-		     const char *icon)
+		     const char *icon,
+		     gulong drive_id,
+		     gulong volume_id)
 {
 	NautilusFile *file;
 
@@ -2905,7 +2979,7 @@ link_info_read_done (NautilusDirectory *directory,
 	directory->details->link_info_read_state = NULL;
 
 	nautilus_file_ref (file);
-	link_info_done (directory, file, uri, name, icon);
+	link_info_done (directory, file, uri, name, icon, drive_id, volume_id);
 	nautilus_file_changed (file);
 
 	if (!should_read_link_info_sync (file)) {
@@ -2924,6 +2998,7 @@ link_info_nautilus_link_read_callback (GnomeVFSResult result,
 {
 	NautilusDirectory *directory;
 	char *buffer, *uri, *name, *icon;
+	gulong drive_id, volume_id;
 
 	directory = NAUTILUS_DIRECTORY (callback_data);
 
@@ -2936,17 +3011,17 @@ link_info_nautilus_link_read_callback (GnomeVFSResult result,
 		uri = NULL;
 		name = NULL;
 		icon = NULL;
+		volume_id = drive_id = 0;
 	} else {
 		/* The libxml parser requires a zero-terminated array. */
 		buffer = g_realloc (file_contents, bytes_read + 1);
 		buffer[bytes_read] = '\0';
-		uri = nautilus_link_get_link_uri_given_file_contents (NULL, buffer, bytes_read);
-                name = nautilus_link_get_link_name_given_file_contents (NULL, buffer, bytes_read);
-                icon = nautilus_link_get_link_icon_given_file_contents (NULL, buffer, bytes_read);
+		nautilus_link_get_link_info_given_file_contents (buffer, bytes_read,
+								 &uri, &name, &icon, &drive_id, &volume_id);
 		g_free (buffer);
 	}
 
-	link_info_read_done (directory, uri, name, icon);
+	link_info_read_done (directory, uri, name, icon, drive_id, volume_id);
 	g_free (uri);
 	g_free (name);
 	g_free (icon);
@@ -3047,7 +3122,7 @@ link_info_start (NautilusDirectory *directory,
 	
 	/* If it's not a link we are done. If it is, we need to read it. */
 	if (!(nautilus_style_link || (is_directory && dot_directory_uri != NULL) )) {
-		link_info_done (directory, file, NULL, NULL, NULL);
+		link_info_done (directory, file, NULL, NULL, NULL, 0, 0);
 	} else if (should_read_link_info_sync (file)) {
 		directory->details->link_info_read_state = g_new0 (LinkInfoReadState, 1);
 		directory->details->link_info_read_state->file = file;
@@ -3085,6 +3160,153 @@ link_info_start (NautilusDirectory *directory,
 	g_free (uri);
 }
 
+static void
+extension_info_cancel (NautilusDirectory *directory)
+{
+	if (directory->details->extension_info_in_progress != NULL) {
+		nautilus_info_provider_cancel_update 
+			(directory->details->extension_info_provider,
+			 directory->details->extension_info_in_progress);
+
+		directory->details->extension_info_in_progress = NULL;
+		directory->details->extension_info_file = NULL;
+		directory->details->extension_info_provider = NULL;
+
+		async_job_end (directory, "extension info");
+	}
+}
+	
+static void
+extension_info_stop (NautilusDirectory *directory)
+{
+	if (directory->details->extension_info_in_progress != NULL) {
+		NautilusFile *file;
+
+		file = directory->details->extension_info_file;
+		if (file != NULL) {
+			g_assert (NAUTILUS_IS_FILE (file));
+			g_assert (file->details->directory == directory);
+			if (is_needy (file, lacks_extension_info, wants_extension_info)) {
+				return;
+			}
+		}
+
+		/* The info is not wanted, so stop it. */
+		extension_info_cancel (directory);
+	}
+}
+
+static void
+finish_info_provider (NautilusDirectory *directory,
+		      NautilusFile *file,
+		      NautilusInfoProvider *provider)
+{
+	file->details->pending_info_providers = 
+		g_list_remove  (file->details->pending_info_providers,
+				provider);
+	g_object_unref (provider);
+
+	nautilus_directory_async_state_changed (directory);
+
+	if (file->details->pending_info_providers == NULL) {
+		nautilus_file_info_providers_done (file);
+	}
+}
+
+
+static gboolean
+info_provider_idle_callback (gpointer user_data)
+{
+	InfoProviderResponse *response;
+	NautilusDirectory *directory;
+
+	response = user_data;
+	directory = response->directory;
+
+	if (response->handle != directory->details->extension_info_in_progress
+	    || response->provider != directory->details->extension_info_provider) {
+		g_warning ("Unexpected plugin response.  This probably indicates a bug in a Nautilus extension: handle=%p", response->handle);
+	} else {
+		NautilusFile *file;
+		async_job_end (directory, "extension info");
+
+		file = directory->details->extension_info_file;
+
+		directory->details->extension_info_file = NULL;
+		directory->details->extension_info_provider = NULL;
+		directory->details->extension_info_in_progress = NULL;
+
+		finish_info_provider (directory, file, response->provider);
+	}
+
+	return FALSE;
+}
+
+static void
+info_provider_callback (NautilusInfoProvider *provider,
+			NautilusOperationHandle *handle,
+			NautilusOperationResult result,
+			gpointer user_data)
+{
+	InfoProviderResponse *response;
+	
+	response = g_new0 (InfoProviderResponse, 1);
+	response->provider = provider;
+	response->handle = handle;
+	response->result = result;
+	response->directory = NAUTILUS_DIRECTORY (user_data);
+
+	g_idle_add (info_provider_idle_callback, response);
+}
+
+static void
+extension_info_start (NautilusDirectory *directory,
+		      NautilusFile *file)
+{
+	NautilusInfoProvider *provider;
+	NautilusOperationResult result;
+	NautilusOperationHandle *handle;
+	GClosure *update_complete;
+
+	if (directory->details->extension_info_in_progress != NULL) {
+		return;
+	}
+	
+	if (!is_needy (file, lacks_extension_info, wants_extension_info)) {
+		return;
+	}
+
+	if (!async_job_start (directory, "extension info")) {
+		return;
+	}
+
+	provider = file->details->pending_info_providers->data;
+
+	update_complete = g_cclosure_new (G_CALLBACK (info_provider_callback),
+					  directory,
+					  NULL);
+	g_closure_set_marshal (update_complete,
+			       nautilus_marshal_VOID__POINTER_ENUM);
+			       
+	result = nautilus_info_provider_update_file_info
+		(provider, 
+		 NAUTILUS_FILE_INFO (file), 
+		 update_complete, 
+		 &handle);
+
+	g_closure_unref (update_complete);
+
+	if (result == NAUTILUS_OPERATION_COMPLETE ||
+	    result == NAUTILUS_OPERATION_FAILED) {
+		finish_info_provider (directory, file, provider);
+		async_job_end (directory, "extension info");
+	} else {
+		directory->details->extension_info_in_progress = handle;
+		directory->details->extension_info_provider = provider;
+		directory->details->extension_info_file = file;
+	}
+}
+
 static void
 start_or_stop_io (NautilusDirectory *directory)
 {
@@ -3100,6 +3322,7 @@ start_or_stop_io (NautilusDirectory *directory)
 	mime_list_stop (directory);
 	top_left_stop (directory);
 	link_info_stop (directory);
+	extension_info_stop (directory);
 
 	/* Take files that are all done off the queue. */
 	while (!nautilus_file_queue_is_empty (directory->details->high_priority_queue)) {
@@ -3126,6 +3349,20 @@ start_or_stop_io (NautilusDirectory *directory)
 			mime_list_start (directory, file);
 			top_left_start (directory, file);
 			return;
+		} else {
+			move_file_to_extension_queue (directory, file);
+
+		}
+	}
+
+	/* Low priority queue must be empty */
+	while (!nautilus_file_queue_is_empty (directory->details->extension_queue)) {
+		file = nautilus_file_queue_head (directory->details->extension_queue);
+
+		if (file_needs_extension_work_done (directory, file)) {
+			/* Start getting attributes if possible */
+			extension_info_start (directory, file);
+			return;
 		} else {
 			nautilus_directory_remove_file_from_work_queue (directory, file);
 
@@ -3178,6 +3415,7 @@ nautilus_directory_cancel (NautilusDirectory *directory)
 	link_info_cancel (directory);
 	mime_list_cancel (directory);
 	top_left_cancel (directory);
+	extension_info_cancel (directory);
 
 	/* We aren't waiting for anything any more. */
 	if (waiting_directories != NULL) {
@@ -3272,6 +3510,10 @@ cancel_loading_attributes (NautilusDirectory *directory,
 	if (request.link_info) {
 		link_info_cancel (directory);
 	}
+
+	if (request.extension_info) {
+		extension_info_cancel (directory);
+	}
 	
 	/* FIXME bugzilla.gnome.org 45064: implement cancelling metadata when we
 	   implement invalidating metadata */
@@ -3356,12 +3598,24 @@ file_needs_low_priority_work_done (NautilusDirectory *directory,
 	return FALSE;
 }
 
+static gboolean
+file_needs_extension_work_done (NautilusDirectory *directory,
+				   NautilusFile *file)
+{
+	if (is_needy (file, lacks_extension_info, wants_extension_info)) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 file_needs_work_done (NautilusDirectory *directory,
 		      NautilusFile *file)
 {
 	return (file_needs_high_priority_work_done (directory, file) ||
-		file_needs_low_priority_work_done (directory, file));
+		file_needs_low_priority_work_done (directory, file) || 
+		file_needs_extension_work_done (directory, file));
 }
 
 
@@ -3401,6 +3655,8 @@ nautilus_directory_remove_file_from_work_queue (NautilusDirectory *directory,
 				    file);
 	nautilus_file_queue_remove (directory->details->low_priority_queue,
 				    file);
+	nautilus_file_queue_remove (directory->details->extension_queue,
+				    file);
 }
 
 
@@ -3408,15 +3664,32 @@ static void
 move_file_to_low_priority_queue (NautilusDirectory *directory,
 				 NautilusFile *file)
 {
-	if (!file_needs_low_priority_work_done (directory, file)) {
-		nautilus_file_queue_remove (directory->details->high_priority_queue,
-					    file);
-		return;
-	}
-
 	/* Must add before removing to avoid ref underflow */
 	nautilus_file_queue_enqueue (directory->details->low_priority_queue,
 				     file);
 	nautilus_file_queue_remove (directory->details->high_priority_queue,
 				    file);
+
+	if (!file_needs_low_priority_work_done (directory, file)) {
+		move_file_to_extension_queue (directory, file);
+
+		return;
+	}
+}
+
+static void
+move_file_to_extension_queue (NautilusDirectory *directory,
+			      NautilusFile *file)
+{
+	if (!file_needs_extension_work_done (directory, file)) {
+		nautilus_file_queue_remove (directory->details->low_priority_queue,
+					    file);
+		return;
+	}
+
+	/* Must add before removing to avoid ref underflow */
+	nautilus_file_queue_enqueue (directory->details->extension_queue,
+				     file);
+	nautilus_file_queue_remove (directory->details->low_priority_queue,
+				    file);
 }
