@@ -28,8 +28,8 @@
 #ifndef NAUTILUS_BOOKMARK_PARSING_H
 #define NAUTILUS_BOOKMARK_PARSING_H
 
-#include <libnautilus-extensions/nautilus-bookmark.h>
-#include <gnome-xml/tree.h>
+#include <libnautilus-private/nautilus-bookmark.h>
+#include <libxml/tree.h>
 
 NautilusBookmark *nautilus_bookmark_new_from_node (xmlNodePtr node);
 
